#!/usr/bin/env python3


import scripter
import config
import hashlib

def open_onikakushi() -> str:

    # Opening files, reading hashes
    try:
        with open(config.get('original_path'), 'rb') as f:
            md5hash = hashlib.file_digest(f, "md5").hexdigest()

        origfile = open(config.get('original_path'), 'r', encoding='shift_jisx0213')

    except:
        print(f"Could not open original script path {config.get('original_path')}")
        exit(1)

    replacementsfile = None
    try:
        replacementsfile = open('replacements/' + md5hash)
    except:
        print(f"The original script file with hash {md5hash} does not have a matching replacement file, no replacements will be executed")

    outpath = 'tmp/onikakushi.txt'
    outfile = open(outpath, 'w', encoding='shift_jisx0213')

    replacements = {}
    exclude = []

    # Parse replacements file
    # Syntax: 
    # - '-<number>' -> exclude this line
    # - Begins with '#' -> comment
    # - '<number>:<string>' -> replace the line in original script with this string
    if replacementsfile != None:
        try:
            for line in replacementsfile:
                if line.startswith('#'):
                    continue
                if line.startswith('-'):
                    exclude.append(int(line[1:]))
                    continue
                split_line = line.split(':', 1) 
                offset_number = int(split_line[0])
                replace_string = split_line[1]
                replacements[offset_number] = replace_string
        except:
            print(f"Malformed replacement file on {md5hash}, line: {line}")
            exit(1)

    for i, line in enumerate(origfile):
        if i not in exclude and i not in replacements:
            outfile.write(fix_common_displaced_commands(line)+'\n')
        else:
            outfile.write(replacements.get(i, line[:-1])+'\n')

    outfile.close()
    origfile.close()

    if replacementsfile != None:
        replacementsfile.close()

    return outpath


# Fix the '「!s*' and '!sd。」@' plague
def fix_common_displaced_commands(line: str) -> str:
    ret = ''
    tokens = scripter.parse_line(line)

    if len(tokens) > 1 and \
       tokens[1].type == scripter.TokenType.COMMAND \
       and tokens[0].token == '　':

        tokens.pop(0)


    # '「!s*'
    if len(tokens) > 2 and \
       tokens[0].token == '「' and tokens[1].token.startswith('!s'):
        # !s to beggining
        tokens[0] = tokens[1]
        # prepend 「 to next token
        tokens[2].token =  '「' + tokens[2].token
        # remove 「 token
        tokens.pop(1)


    # '!sd。」'
    if (
        len(tokens) > 3
        and tokens[-3].token == '!sd'
        and tokens[-2].token in [ # quot
            '。」',
            '」',
            '。',
            '、」',
        ]
        and tokens[-1].type == scripter.TokenType.COMMAND # final
    ):
        # Get the final symbols
        # [text][command]<[text][command]>
        quot = tokens[-2].token
        final = tokens[-1].token
        # Append them to the previous ones (-3 and -4)
        # <[text+text][command+command]>[text][command]
        tokens[-4].token += quot
        tokens[-3].token += final
        # Trim the leftovers
        # [text+text][command+command]<>
        tokens = tokens[:-2]


    for t in tokens:
        ret += t.token

    return ret
